from django.utils.crypto import get_random_string
from rest_framework import serializers
from django.contrib.auth import get_user_model
from .models import Account, Session, VerficationToken
from .signals import user_created, user_updated

User = get_user_model()


class AdapterUserSerializer(serializers.ModelSerializer):
    name = serializers.CharField(required=False, allow_null=True)
    image = serializers.URLField(required=False, allow_null=True)
    emailVerified = serializers.DateTimeField(
        required=False, allow_null=True, source="email_verified"
    )

    class Meta:
        model = User
        fields = ["id", "email", "emailVerified", "name", "image"]

    def get_names_pair(self, name: str) -> tuple[str, str]:
        name = name.split(" ")
        if len(name) == 2:
            return (name[0], name[1])
        elif len(name) == 1:
            return (name[0], "")
        else:
            return ("", "")

    def create(self, validated_data: dict) -> User:
        name = validated_data.pop("name", None)
        image = validated_data.pop("image", None)
        if name is not None:
            (
                validated_data["first_name"],
                validated_data["last_name"],
            ) = self.get_names_pair(name)

        instance = super().create(validated_data)
        if name is not None:
            instance.name = validated_data.get("name")
        if image is not None:
            instance.image = image

        validated_data["password"] = get_random_string(32)
        user_created.send(
            sender=self.__class__,
            user=instance,
            validated_data=validated_data,
            image=image,
            name=name,
        )

        return AdapterPublicUserSerializer(instance).data

    def update(self, instance: User, validated_data):
        if validated_data.get("name", None) is None:
            instance.name = instance.get_full_name()
        else:
            (
                validated_data["first_name"],
                validated_data["last_name"],
            ) = self.get_names_pair(validated_data.get("name"))
        user_updated.send(
            sender=self.__class__, user=instance, validated_data=validated_data
        )
        return super().update(instance, validated_data)


class AdapterPublicUserSerializer(AdapterUserSerializer):
    name = serializers.CharField(source="get_full_name")
    image = serializers.SerializerMethodField(method_name="get_profile_image")
    emailVerified = serializers.CharField(source="email_verified")

    def get_profile_image(self, obj: User) -> str:
        if hasattr(obj, "get_image") and callable(getattr(obj, "get_image")):
            img = getattr(obj, "get_image")()
            return img if isinstance(img, str) else ""
        return ""

    class Meta:
        model = User
        fields = ["id", "email", "emailVerified", "name", "image"]


class LinkAccountSerializer(serializers.ModelSerializer):
    providerAccountId = serializers.CharField(source="provider_account_id")
    userId = serializers.CharField(source="user_id")

    class Meta:
        model = Account
        fields = [
            "id",
            "userId",
            "type",
            "provider",
            "providerAccountId",
            "refresh_token",
            "access_token",
            "expires_at",
            "token_type",
            "scope",
            "id_token",
            "session_state",
        ]


class RemoteAdapterSessionSerializer(serializers.ModelSerializer):
    sessionToken = serializers.CharField(source="session_token")
    userId = serializers.CharField(source="user_id")

    class Meta:
        model = Session
        fields = [
            "id",
            "userId",
            "sessionToken",
            "expires",
        ]


class RemoteAdapterSessionAndUserSerializer(serializers.Serializer):
    session = RemoteAdapterSessionSerializer()
    user = AdapterPublicUserSerializer()


class RemoteAdapterVerificationTokensSerializers(serializers.ModelSerializer):
    class Meta:
        model = VerficationToken
        fields = "__all__"



from rest_framework.response import Response as DRFResponse
from typing import Any
from rest_framework import status

SendableJson = dict | Any | None


class Response:
    @staticmethod
    def get_response_dict(
        data: dict, status: int, message: str, error: SendableJson = None
    ):
        return {"data": data, "status": status, "message": message, "error": error}

    @staticmethod
    def success(
        data: SendableJson = None,
        status: int = status.HTTP_200_OK,
        template_name: Any | None = None,
        headers: Any | None = None,
        exception: bool = False,
        content_type: Any | None = None,
    ):
        return DRFResponse(
            data=Response.get_response_dict(
                data=data, status=status, message="success"
            ),
            status=status,
            template_name=template_name,
            headers=headers,
            exception=exception,
            content_type=content_type,
        )

    @staticmethod
    def error(
        data: SendableJson = None,
        status: int = status.HTTP_400_BAD_REQUEST,
        template_name: Any | None = None,
        headers: Any | None = None,
        exception: bool = False,
        content_type: Any | None = None,
        error: SendableJson = None,
    ):
        return DRFResponse(
            data=Response.get_response_dict(
                data=data, status=status, message="error", error=error
            ),
            status=status,
            template_name=template_name,
            headers=headers,
            exception=exception,
            content_type=content_type,
        )

    @staticmethod
    def get_response_schema(schema: dict) -> dict:
        return {
            "type": "object",
            "properties": {
                "data": schema,
                "status": {
                    "type": "integer",
                    "format": "int32",
                },
                "message": {
                    "type": "string",
                },
                "error": {
                    "type": "object",
                    "nullable": True,
                },
            },
        }



from django.conf import settings as django_settings

settings = getattr(django_settings, "DJANGO_NEXT_AUTH_ADAPTER", {})

permission_classes = settings.get("PERMISSION_CLASSES")
remote_auth_token = settings.get("REMOTE_AUTH_TOKEN", None)



from . import views
from django.urls import path

urlpatterns = [
    # next auth http adapter custom methods
    path("signup/", views.RemoteAdapterSignupView.as_view(), name="signup"),
    path("get-user/<uuid:id>/", views.RetrieveUserByIdView.as_view(), name="get_user"),
    path(
        "get-user-by-email/<str:email>/",
        views.RetrieveUserByEmailAPIView.as_view(),
        name="get_user_by_email",
    ),
    path(
        "get-user-by-account/<str:provider>/<str:account_id>/",
        views.RetrieveUserByAccountAPIView.as_view(),
        name="get_user_by_account",
    ),
    path("update-user/", views.UpdateUserAPIView.as_view(), name="update_user"),
    path("delete-user/<str:id>/", views.DeletUserAPIView.as_view(), name="delete_user"),
    path("link-account/", views.LinkAccountAPIView.as_view(), name="link_account"),
    path(
        "unlink-account/<str:provider>/<str:account_id>/",
        views.UnlinkAccountAPIView.as_view(),
        name="unlink_account",
    ),
    path(
        "create-session/", views.CreateSessionAPIView.as_view(), name="create_session"
    ),
    path(
        "get-session-and-user/<str:session_token>",
        views.GetSessionAndUserAPIView.as_view(),
        name="get_session_and_user",
    ),
    path(
        "update-session/", views.UpdateSessionAPIView.as_view(), name="update_session"
    ),
    path(
        "delete-session/", views.DeleteSessionAPIView.as_view(), name="delete_session"
    ),
    path(
        "create-verification-token/",
        views.CreateVerificationTokenAPIView.as_view(),
        name="create_verification_token",
    ),
    path(
        "use-verification-token/",
        views.RetrieveVerificationToken.as_view(),
        name="use_verification_token",
    ),
]



from uuid import uuid4
from typing import Type
from django.db import models
from django.contrib.auth import get_user_model

User: Type[get_user_model()] = get_user_model()


def generate_db_id():
    return str(uuid4())


class Account(models.Model):
    id = models.CharField(
        primary_key=True, default=generate_db_id, editable=False, max_length=36
    )
    user: User = models.ForeignKey(
        to=User,
        related_name="accounts",
        on_delete=models.CASCADE,
    )
    type = models.CharField(max_length=50)
    provider = models.CharField(max_length=50)
    provider_account_id = models.CharField(max_length=255)
    refresh_token = models.TextField(blank=True, null=True)
    access_token = models.TextField(blank=True, null=True)
    expires_at = models.PositiveIntegerField(blank=True, null=True)
    token_type = models.CharField(max_length=50, blank=True, null=True)
    scope = models.CharField(max_length=255, blank=True, null=True)
    id_token = models.TextField(blank=True, null=True)
    session_state = models.CharField(max_length=255, blank=True, null=True)

    class Meta:
        db_table = "accounts"
        verbose_name = "account"
        verbose_name_plural = "accounts"
        unique_together = ("provider", "provider_account_id")

    def __str__(self):
        return f"{self.id} - {self.user.email}"


class Session(models.Model):
    id = models.CharField(
        primary_key=True, default=generate_db_id, editable=False, max_length=36
    )
    user: User = models.ForeignKey(
        to=User,
        related_name="sessions",
        on_delete=models.CASCADE,
    )
    session_token = models.CharField(max_length=255, unique=True)
    expires = models.DateTimeField()

    class Meta:
        db_table = "sessions"
        verbose_name = "session"
        verbose_name_plural = "sessions"

    def __str__(self):
        return self.id


class VerficationToken(models.Model):
    identifier = models.CharField(max_length=255)
    token = models.CharField(max_length=255, unique=True)
    expires = models.DateTimeField()

    class Meta:
        db_table = "verification_tokens"
        verbose_name = "verification token"
        verbose_name_plural = "verification tokens"
        unique_together = ("identifier", "token")



from rest_framework import status
from rest_framework.response import Response as DRFResponse
from rest_framework import generics
from rest_framework.views import APIView
from .mixins import WrappedResponseMixin
from django.contrib.auth import get_user_model
from .settings import permission_classes as default_permission_classes
from .permissions import AllowRemoteAuthServer
from .models import Account, Session, VerficationToken

from . import serializers as auth_serializers

User = get_user_model()
permission_classes = default_permission_classes or [AllowRemoteAuthServer]


class RemoteAdapterSignupView(WrappedResponseMixin, APIView):
    permission_classes = permission_classes
    serializer_class = auth_serializers.AdapterUserSerializer
    queryset = User.objects.all()

    def post(self, request, *args, **kwargs):
        try:
            user = User.objects.get(email=request.data.get("email"))
            if user is not None:
                return DRFResponse(
                    data=auth_serializers.AdapterPublicUserSerializer(user).data,
                    status=status.HTTP_200_OK,
                )
        except User.DoesNotExist:
            pass
        serializer = self.serializer_class(data=request.data)
        serializer.is_valid(raise_exception=True)
        res = serializer.create(validated_data=serializer.validated_data)
        return DRFResponse(data=res, status=status.HTTP_201_CREATED)


class RetrieveUserByIdView(WrappedResponseMixin, generics.RetrieveAPIView):
    permission_classes = permission_classes
    serializer_class = auth_serializers.AdapterPublicUserSerializer
    queryset = User.objects.all()

    def get_object(self):
        return self.queryset.get(id=self.kwargs.get("id"))


class RetrieveUserByEmailAPIView(WrappedResponseMixin, generics.RetrieveAPIView):
    permission_classes = permission_classes
    serializer_class = auth_serializers.AdapterPublicUserSerializer
    queryset = User.objects.all()

    def get_object(self):
        return self.queryset.get(email=self.kwargs.get("email"))

    def handle_exception(self, exc):
        return DRFResponse(data=str(exc), status=status.HTTP_400_BAD_REQUEST)


class RetrieveUserByAccountAPIView(WrappedResponseMixin, generics.RetrieveAPIView):
    permission_classes = permission_classes
    serializer_class = auth_serializers.AdapterPublicUserSerializer
    queryset = Account.objects.all()

    def get_object(self):
        account_id = self.kwargs.get("account_id")
        provider = self.kwargs.get("provider")
        return self.queryset.get(provider_account_id=account_id, provider=provider).user

    def handle_exception(self, exc):
        return DRFResponse(data=str(exc), status=status.HTTP_400_BAD_REQUEST)


class UpdateUserAPIView(WrappedResponseMixin, generics.UpdateAPIView):
    permission_classes = permission_classes
    serializer_class = auth_serializers.AdapterUserSerializer
    queryset = User.objects.all()

    def get_object(self):
        return self.queryset.get(email=self.request.data.get("email"))

    def handle_exception(self, exc):
        return DRFResponse(data=str(exc), status=status.HTTP_400_BAD_REQUEST)


class DeletUserAPIView(WrappedResponseMixin, generics.DestroyAPIView):
    permission_classes = permission_classes
    serializer_class = auth_serializers.AdapterUserSerializer
    queryset = User.objects.all()
    lookup_url_kwarg = "id"


class LinkAccountAPIView(WrappedResponseMixin, generics.CreateAPIView):
    permission_classes = permission_classes
    serializer_class = auth_serializers.LinkAccountSerializer
    queryset = Account.objects.all()


class UnlinkAccountAPIView(WrappedResponseMixin, generics.DestroyAPIView):
    permission_classes = permission_classes
    serializer_class = auth_serializers.LinkAccountSerializer
    queryset = Account.objects.all()

    def get_object(self):
        provider = self.request.data.get("provider")
        provider_account_id = self.request.data.get("providerAccountId")
        return self.queryset.get(
            provider=provider, provider_account_id=provider_account_id
        )


class CreateSessionAPIView(WrappedResponseMixin, generics.CreateAPIView):
    permission_classes = permission_classes
    serializer_class = auth_serializers.RemoteAdapterSessionSerializer
    queryset = User.objects.all()


class UpdateSessionAPIView(WrappedResponseMixin, generics.UpdateAPIView):
    permission_classes = permission_classes
    serializer_class = auth_serializers.RemoteAdapterSessionSerializer
    queryset = Session.objects.all()

    def get_object(self):
        return self.queryset.get(session_token=self.request.data.get("sessionToken"))

    def handle_exception(self, exc):
        return DRFResponse(data=str(exc), status=status.HTTP_400_BAD_REQUEST)


class GetSessionAndUserAPIView(WrappedResponseMixin, generics.RetrieveAPIView):
    permission_classes = permission_classes
    serializer_class = auth_serializers.RemoteAdapterSessionAndUserSerializer
    queryset = Session.objects.all()

    def get_object(self):
        session = self.queryset.get(session_token=self.kwargs.get("session_token"))
        return {"user": session.user, "session": session}

    def handle_exception(self, exc):
        return DRFResponse(data=str(exc), status=status.HTTP_400_BAD_REQUEST)


class DeleteSessionAPIView(WrappedResponseMixin, generics.DestroyAPIView):
    permission_classes = permission_classes
    serializer_class = auth_serializers.RemoteAdapterSessionSerializer
    queryset = Session.objects.all()

    def get_object(self):
        return self.queryset.get(session_token=self.kwargs.get("session_token"))

    def handle_exception(self, exc):
        return DRFResponse(data=str(exc), status=status.HTTP_400_BAD_REQUEST)


class CreateVerificationTokenAPIView(WrappedResponseMixin, generics.CreateAPIView):
    permission_classes = permission_classes
    serializer_class = auth_serializers.RemoteAdapterVerificationTokensSerializers
    queryset = VerficationToken.objects.all()


class RetrieveVerificationToken(
    WrappedResponseMixin, generics.RetrieveAPIView, generics.CreateAPIView
):
    permission_classes = permission_classes
    serializer_class = auth_serializers.RemoteAdapterVerificationTokensSerializers
    queryset = VerficationToken.objects.all()

    def get(self, request, *args, **kwargs):
        return super().get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        return self.get(request, *args, **kwargs)

    def get_object(self):
        return self.queryset.get(
            identifier=self.request.data.get("identifier"),
            tokne=self.request.data.get("token"),
        )

    def handle_exception(self, exc):
        return super().handle_exception(exc)



from rest_framework.permissions import BasePermission
from .settings import remote_auth_token


class AllowRemoteAuthServer(BasePermission):
    """
    Allow remote auth server to access this API.
    """

    def has_permission(self, request, view):
        token = request.META.get("HTTP_AUTHORIZATION", None)
        if token is None:
            return False

        if token != remote_auth_token:
            return False

        return True



from django.http import HttpRequest
from .response import Response
from rest_framework.response import Response as DRFResponse


class WrappedResponseMixin:
    def is_success_status_code(self, status_code):
        return status_code >= 200 and status_code < 300

    def finalize_response(
        self, request: HttpRequest, response: DRFResponse, *args, **kwargs
    ):
        if self.is_success_status_code(response.status_code):
            wrappend_response = Response.success(
                data=response.data, status=response.status_code
            )
        else:
            wrappend_response = Response.error(
                error=response.data, status=response.status_code
            )
        return super().finalize_response(request, wrappend_response, *args, **kwargs)



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open("README.md") as readme_file:
    readme = readme_file.read()

# with open("HISTORY.md") as history_file:
#     history = history_file.read()

requirements = [
    "Django>=3.0.0",
    "djangorestframework>=3.0",
]

test_requirements = [
    "pytest>=3",
]

setup(
    author="Abdullah Adeel",
    author_email="contact.abdadeel@gamil.com",
    python_requires=">=3.7",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Framework :: Django",
        "Framework :: Django :: 2.0",
        "Framework :: Django :: 2.1",
        "Framework :: Django :: 2.2",
        "Framework :: Django :: 3.0",
        "Framework :: Django :: 3.1",
        "Framework :: Django :: 3.2",
        "Framework :: Django :: 4.0",
    ],
    description="A Django app to to handle server side requirests of next-auth-http-adapter",
    entry_points={},
    install_requires=requirements,
    license="MIT license",
    # long_description=readme + "\n\n" + history,
    long_description=readme,
    long_description_content_type="text/markdown",
    include_package_data=True,
    keywords=[
        "django_next_auth_adapter",
        "django",
        "next-auth",
        "auth",
        "rest",
        "restframework",
    ],
    name="django-next-auth-adapter",
    packages=find_packages(
        include=["django_next_auth_adapter", "django_next_auth_adapter.*"]
    ),
    test_suite="tests",
    tests_require=test_requirements,
    url="https://github.com/mabdullahadeel/django-next-auth-adapter",
    version="0.0.4",
    zip_safe=False,
)
